@@ -1445,6 +1445,10 @@ def load():
             )
         elif len(classes) == 0:
             logger.error("Error retrieving experiment class")
+            if not module_is_initialized(experiment):
+                logger.error(
+                    "The experiment module is only partly initialized. Maybe you have a circular import?"
+                )
             raise (
                 first_err
                 or second_err
@@ -1457,6 +1461,20 @@ def load():
         raise
 
 
+def module_is_initialized(module):
+    """
+    Checks whether a given module has been initialized by catching the AttributeError that happens when accessing
+    an unknown attribute within that module. This is a bit of a hack, but it seems to be the easiest
+    way of checking the modules initialization status.
+    """
+    try:
+        module.abcdefghijklmnop123456789
+    except AttributeError as err:
+        if "partially initialized module" in str(err):
+            return False
+    return True
+
+
 EXPERIMENT_TASK_REGISTRATIONS = []
 
 
@@ -1467,7 +1485,7 @@ def scheduled_task(trigger, **kwargs):
     The task registration is deferred until clock server setup to allow tasks to be
     overridden by subclasses.
 
-    :param trigger: an ``appscheduler`` trigger type. One of "interval", "cron",
+    :param trigger: an ``apscheduler`` trigger type. One of "interval", "cron",
                     or "date"
     :param \**kwargs: other arguments for `apscheduler.schedulers.base.BaseSchedule.scheduled_job`
                       generally used for trigger arguments to determine
